#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 16 08:40:33 2022

@author: richard
"""
from faker.providers import BaseProvider


class BioProvider(BaseProvider):
    """
     Base class of some bio providers
    """

    def __init__(self, generator):
        super().__init__(generator)

    def _dict_leaves(self, data: dict, leaves=None) -> None:
        if leaves is None:
            leaves = []
        nodes = data.keys()
        for key in nodes:
            subnode = data[key]
            if len(subnode) > 0:
                self._dict_leaves(subnode, leaves)
            else:
                leaves.append(key)

    def _leaves_by_category(self, data: dict, category: str) -> list:
        leaves = []
        self._dict_leaves(data[category], leaves)
        return leaves

    def _dict_all(self, data: dict, leaves=None) -> None:
        """
          Gets all keys in nested dict
         """
        if leaves is None:
            leaves = []
        nodes = data.keys()
        for key in nodes:
            subnode = data[key]
            if len(subnode) > 0:
                leaves.append(key)
                self._dict_leaves(subnode, leaves)
            else:
                leaves.append(key)



import random
import faker_biology.bioseq.bioseq_data as dna_data
from faker.providers import BaseProvider


class Bioseq(BaseProvider):
    """
     Provider of DNA / RNA / protein sequences.

    """

    def __init__(self, generator):
        super().__init__(generator)

    def dna(self, length: int = 80) -> str:
        """
        Gets a random DNA sequence. Each nucleotide has equal probability of occurrence
        Parameters
        ----------
        length : int, optional
            Desired length of sequence. The default is 80.
        Returns
        -------
        str
            DNA string.
        """
        return self._seq(length, dna_data.unambiguous_dna_letters)

    def rna(self, length: int = 80) -> str:
        """
        Gets a random RNA sequence. Each nucleotide has equal probability of occurrence
        Parameters
        ----------
        length : int, optional
            Desired length of sequence. The default is 80.
        Returns
        -------
        str
            RNA string.
        """
        return self._seq(length, dna_data.unambiguous_rna_letters)

    def stop_codon(self) -> str:
        """
        A randomly-chosen stop codon from the 3 standard stop codons.

        Returns
        -------
        str
            A stop codon.
        """
        return self.random_element(dna_data.stop_codons)

    def cds(self, length: int = 20) -> str:
        """
        Returns a DNA sequence that will encode a polypeptide. The sequence will
        always begin with 'ATG' and end with a randomly-chosen termination codon.

        e.g. cds(2) could return 'ATGGAAGTCTGA'
        Parameters
        ----------
        length : int, optional
            Number of codons, excluding initial ATG and final termination codon. The default is 20.
        Returns
        -------
        str
            A DNA coding sequence.
        """
        i = 0
        codons = []
        while i < length:
            triplet = self.dna(3)
            if triplet not in dna_data.stop_codons:
                codons.append(triplet)
                i = i + 1

        return "ATG" + "".join(codons) + self.stop_codon()

    def protein(self, length: int = 20) -> str:
        """
        Generates a random protein sequence starting with 'M' and then <length> amino acids
        in single-letter code.

        Parameters
        ----------
        length : int, optional
            Length of sequence after initial 'M'. The default is 20.

        Returns
        -------
        str
            A Protein sequence.

        """
        return "M" + self._seq(length, dna_data.protein_letters)

    def protein_name(self) -> str:
        """
        A randomly chosen protein name from a list of 500 SwissProt protein descriptions
        Returns
        -------
        str
            A name of a real protein.
        """
        return self.random_element(list(dna_data.protein_names.keys()))

    def protein_desc(self) -> str:
        """
        A randomly chosen protein description from a list of 500 SwissProt protein descriptions
        Returns
        -------
        str
            A description of a real protein.
        """
        return self.random_element(list(dna_data.protein_names.values()))

    def protein_name_desc(self) -> dict:
        """
        Returns a name-description pair from a list of 500 SwissProt proteins
        Returns
        -------
        dict
            A name,description tuple.
        """
        index = random.randint(0, len(dna_data.protein_names) - 1)
        key = list(dna_data.protein_names.keys())[index]
        return key, dna_data.protein_names[key]

    def amino_acid(self) -> dna_data.AminoAcid:
        """
        A randomly chosen amino acid from a list of 20 amino acids
        Returns
        -------
        AminoAcid
            A named tuple containing all information about an amino acid.
        """

        return self.random_element(dna_data.amino_acids)

    def amino_acid_name(self) -> str:
        """
        A randomly chosen long amino acid name from a list of 20 amino acids
        Returns
        -------
        str
            A long name of an amino acid.
        """

        return self.random_element(dna_data.amino_acids).full_name

    def amino_acid_3_letters(self) -> str:
        """
        A randomly chosen 3 letters abbreviation for amino acid name from a list of 20 amino acids
        Returns
        -------
        str
            A 3 letters abbreviation for amino acid name.
        """

        return self.random_element(dna_data.amino_acids).three_letters_name

    def amino_acid_1_letter(self) -> str:
        """
        A randomly chosen 1 letter abbreviation for amino acid name from a list of 20 amino acids
        Returns
        -------
        str
            A 1 letter abbreviation for amino acid name.
        """

        return self.random_element(dna_data.amino_acids).one_letter_name

    def amino_acid_mass(self) -> int:
        """
        A randomly chosen amino acid mass from a list of 20 amino acids
        Returns
        -------
        int
            An amino acid mass.
        """

        return self.random_element(dna_data.amino_acids).mass

    def _seq(self, length: int, alphabet: str) -> str:
        alphabet_length = len(alphabet) - 1
        seq = []
        for i in range(length):
            j = self.generator.random.randint(0, alphabet_length)
            seq.append(alphabet[j])
        return "".join(seq)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:20:19 2022

@author: richard
"""
from typing import Sequence
from faker.providers import BaseProvider
from faker_biology import BioProvider

from faker_biology.physiology.celltype_data import cell_types
from faker_biology.physiology.organs_data import organ_data
import faker_biology.physiology.organelles as organelle_data

class Organelle(BaseProvider):
    def __init__(self, generator):
        super().__init__(generator)
        
    def _all(self):
        maj = [x for x in organelle_data.eukaryote_organelles['eukaryotes_major']]
        minor = [x  for x in organelle_data.eukaryote_organelles['eukaryotes_minor']]
        maj.extend(minor)
        return maj
    
    
    def _filtered(self, terms):
        orgs = self._all()
        filtered = [x['name'] for x in  filter(lambda x:x['distribution'] in terms, orgs )]
        return filtered
        
    def common_eukaryotic_organelle(self) -> str:
        """
        Gets an organelle name that appears in 'most' or 'all' organisms

        Returns
        -------
        A string

        """
        common = ['all', 'most']
        filtered = self._filtered(common)
        return self.random_element(filtered)
    
    def plant_organelle(self) -> str:
        """
        Gets an organelle present in plant cells
        Returns
        -------
        str
        """
        terms = ['all','most', 'plants']
        filtered = self._filtered(terms)
        return self.random_element(filtered)
    
    def animal_organelle(self) -> str:
        """
        Gets an organelle present in animal cells
        Returns
        -------
        str
        """
        terms = ['all', 'most', 'animals']
        filtered = self._filtered(terms)
        return self.random_element(filtered)
        
    def organelle(self) -> str:
        """
        A randomly selected cell organelle. Source Wikipedia 
        https://en.wikipedia.org/wiki/Organelle
        Returns
        -------
        str
            An organelle, e.g. 'nucleus'.

        """
        orgs = [x['name'] for x in self._all()]
        return self.random_element(orgs)

    

class CellType(BioProvider):
    """
     Provider of human cell type names. Source of data is Wikipedia:
         https://en.wikipedia.org/wiki/List_of_distinct_cell_types_in_the_adult_human_body
             
    """

    def __init__(self, generator):
        super().__init__(generator)

    def categories(self) -> Sequence[str]:
        """
        A list of cell-type categories
        """
        return list(cell_types.keys())

    def celltype(self) -> str:
        """
        Gets a random human cell type
        """
        leaves = []
        self._dict_leaves(cell_types, leaves)
        return self.random_element(leaves)


class Organ(BioProvider):
    """
     Provider of human organ names. Source of data is Wikipedia:
         https://en.wikipedia.org/wiki/List_of_organs_of_the_human_body
             
    """

    def __init__(self, generator):
        super().__init__(generator)

    def categories(self) -> Sequence[str]:
        """
        A list of organ categories
        """
        return list(organ_data.keys())

    def organ(self) -> str:
        """
        Gets a random mammalian organ
        """
        leaves = []
        self._dict_leaves(organ_data, leaves)
        return self.random_element(leaves)

    def non_reproductive_organ(self) -> str:
        """
        Gets a random non-reproductive organ
        """
        return self.random_element(self._generate_non_repr_organs())

    def _generate_non_repr_organs(self) -> list:
        if not hasattr(self, "_nonr_items"):
            categories = filter(lambda c: "eproductive" not in c, self.categories())
            self._nonr_items = []
            for c in categories:
                leaves_by_c = self._leaves_by_category(organ_data, c)
                # print (f" leaves by c = {leaves_by_c}")
                self._nonr_items.extend(leaves_by_c)
        return self._nonr_items



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:27:03 2022

@author: richard
"""

import unittest
from faker import Faker
from faker_biology.physiology import CellType


fake = Faker()
fake.add_provider(CellType)
cell_provider = CellType(None)


class OrganTest(unittest.TestCase):
    def test_celltype_categories(self):
        self.assertEqual(3, len(cell_provider.categories()))

    def test_random_cell_type(self):
        cells = set([fake.unique.celltype() for i in range(15)])
        self.assertEqual(15, len(cells))

    def test_dict_leaves_default_arg_not_shared(self):
        # Mutable default bug: calling _dict_leaves twice with an explicit list
        # each time should give the same results. This verifies that a fresh list
        # is used each time rather than a shared default accumulating state.
        from faker_biology import BioProvider
        p = BioProvider(None)
        data = {"a": {}, "b": {}, "c": {}}

        first = []
        p._dict_leaves(data, first)

        second = []
        p._dict_leaves(data, second)

        self.assertEqual(first, second,
            "Repeated calls to _dict_leaves should return identical results")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:27:03 2022

@author: richard
"""

import unittest
from faker import Faker
from faker_biology.mol_biol import RestrictionEnzyme


fake = Faker()
fake.add_provider(RestrictionEnzyme)
re_provider = RestrictionEnzyme(None)


class ReTest(unittest.TestCase):
    def test_random_re(self):
        res = [fake.unique.restriction_enzyme() for i in range(100)]
        self.assertEqual(100, len(res))
        self.assertTrue(all([isinstance(re, str) for re in res]))

    def test_random_redata(self):
        res = [fake.restriction_enzyme_data() for i in range(100)]
        self.assertTrue(all([isinstance(re, dict) for re in res]))

    def test_min_size(self):
        res = [fake.restriction_enzyme(8) for i in range(20)]
        self.assertTrue(all([isinstance(re, str) for re in res]))

    def test_blunt(self):
        res = [fake.unique.blunt() for i in range(50)]
        self.assertTrue(all([isinstance(re, str) for re in res]))

    def test_rest_dict_not_loaded_until_first_use(self):
        RestrictionEnzyme._rest_dict = None  # reset
        self.assertIsNone(RestrictionEnzyme._rest_dict)
        fake.restriction_enzyme()
        self.assertIsNotNone(RestrictionEnzyme._rest_dict)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:27:03 2022

@author: richard
"""

import unittest

from faker import Faker

from faker_biology.bioseq import Bioseq, bioseq_data


fake = Faker()
fake.add_provider(Bioseq)
re_provider = Bioseq(None)


class BioseqTest(unittest.TestCase):
    def test_random_dna(self):
        dna_seq = fake.dna(200)
        self.assertTrue(i in dna_seq for i in "ATCG")
        rna_seq = fake.rna(200)
        self.assertTrue(i in rna_seq for i in "AUCG")

    def test_cds(self):
        cds = fake.cds(20)
        self.assertEqual(66, len(cds))
        self.assertTrue(cds.startswith("ATG"))
        self.assertTrue(cds[-3:] in bioseq_data.stop_codons)

    def test_cds_respects_length(self):
        # ATG (3) + length * 3 codons + stop codon (3) = (length + 2) * 3
        for length in [5, 10, 50]:
            cds = fake.cds(length)
            expected_len = (length + 2) * 3
            self.assertEqual(expected_len, len(cds), f"cds({length}) should have length {expected_len}, got {len(cds)}")
            self.assertTrue(cds.startswith("ATG"))
            self.assertTrue(cds[-3:] in bioseq_data.stop_codons)

    def test_protein(self):
        protein = fake.protein(20)
        self.assertEqual(21, len(protein))
        self.assertTrue(protein.startswith("M"))

    def test_protein_name(self):
        protein = fake.protein_name()

    def test_protein_name_desc_no_index_error(self):
        # random.randint upper bound is inclusive, so calling this many times
        # should eventually trigger an IndexError if the bound is off-by-one
        for _ in range(1000):
            name, desc = fake.protein_name_desc()
            self.assertIsInstance(name, str)
            self.assertIsInstance(desc, str)

    def test_amino_acid_name(self):
        amino_acid_names = [
            "Alanine",
            "Arginine",
            "Asparagine",
            "Aspartic Acid",
            "Cysteine",
            "Glutamic Acid",
            "Glutamine",
            "Glycine",
            "Histidine",
            "Isoleucine",
            "Leucine",
            "Lysine",
            "Methionine",
            "Phenylalanine",
            "Proline",
            "Serine",
            "Threonine",
            "Tryptophan",
            "Tyrosine",
            "Valine",
        ]

        amino_acid = fake.amino_acid_name()
        self.assertIn(amino_acid, amino_acid_names)

    def test_amino_acid_3_letters(self):
        amino_acid_names = [
            "Ala",
            "Arg",
            "Asn",
            "Asp",
            "Cys",
            "Glu",
            "Gln",
            "Gly",
            "His",
            "Ile",
            "Leu",
            "Lys",
            "Met",
            "Phe",
            "Pro",
            "Ser",
            "Thr",
            "Trp",
            "Tyr",
            "Val",
        ]

        amino_acid = fake.amino_acid_3_letters()
        self.assertIn(amino_acid, amino_acid_names)

    def test_amino_acid_1_letter(self):
        amino_acid_names = "ARNDCEQGHILKMFPSTWYV"

        amino_acid = fake.amino_acid_1_letter()
        self.assertIn(amino_acid, amino_acid_names)

    def test_amino_acid(self):
        amino_acid = fake.amino_acid()
        self.assertIn(amino_acid, bioseq_data.amino_acids)

    def test_amino_mass(self):
        masses = [71, 156, 114, 115, 103, 129, 128, 57, 137, 113, 113, 128, 131, 147, 97, 87, 101, 186, 163, 99]
        amino_acid_mass = fake.amino_acid_mass()
        self.assertIn(amino_acid_mass, masses)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:27:03 2022

@author: richard
"""

import unittest
from faker import Faker
from faker_biology.taxonomy import ModelOrganism
from faker_biology.taxonomy.taxonomy_data import MODEL_ORGANISMS_ENGLISH, MODEL_ORGANISMS_LATIN


fake = Faker()
fake.add_provider(ModelOrganism)


class ModelOrganismTest(unittest.TestCase):
    def test_model_organism_latin(self):
        self.assertIn(fake.organism_latin(), MODEL_ORGANISMS_LATIN)

    def test_model_organism_e(self):
        self.assertIn(fake.organism_english(), MODEL_ORGANISMS_ENGLISH)

    def test_model_organism(self):
        o = fake.organism()
        self.assertIn(o.english, MODEL_ORGANISMS_ENGLISH)
        self.assertIn(o.latin, MODEL_ORGANISMS_LATIN)
            



from typing import Sequence

from faker.providers import BaseProvider
from faker_biology import BioProvider

import faker_biology.mol_biol.antibody as ab
from faker_biology.mol_biol.enzyme_data import enzymes


class Antibody(BaseProvider):
    """
     Provider of fake antibody data. 
    """

    def __init__(self, generator):
        super().__init__(generator)

    def antibody_isotype(self) -> str:
        """
        A random isotype like IgG, IgM etc
        Returns
        -------
        str
        """
        return self.random_element(ab.ANTIBODY_ISOTYPE)

    def antibody_application(self) -> str:
        """
        A random application of an antibody, e.g. ELISA, Western Blot
        Returns
        -------
        str
        """
        return self.random_element(ab.ANTIBODY_APPLICATION)

    def antibody_source(self) -> str:
        """
        A random name of an organism used to generate antibodies
        Returns
        -------
        str
        """
        return self.random_element(ab.ANTIBODY_SOURCE)

    def dilution(self) -> str:
        """
        A random antibody dilution in format 1:X
        Returns
        -------
        str
        """
        return self.random_element(ab.DILUTIONS)


class RestrictionEnzyme(BaseProvider):
    """
     Restriction enzyme: source is https://github.com/biopython/biopython/blob/master/Bio/Restriction/Restriction_Dictionary.py
     which in turn is derived from REBASE emboss files version 112 (2021).
    """

    _rest_dict = None

    @classmethod
    def _get_rest_dict(cls) -> dict:
        if cls._rest_dict is None:
            from faker_biology.mol_biol.re import rest_dict
            cls._rest_dict = rest_dict
        return cls._rest_dict

    def __init__(self, generator):
        super().__init__(generator)

    def restriction_enzyme(self, min_length=0) -> str:
        """
         Returns name of a restriction enzyme. Can choose enzymes cutting above a particular length
         Raises ValueError if no enzymes match search criteria
        """
        keys = []
        if min_length > 0:
            for k, v in self._get_rest_dict().items():
                if v["size"] >= min_length:
                    keys.append(k)
        else:
            keys = self._get_rest_dict().keys()
        if len(keys) == 0:
            raise ValueError("No enzymes were found matching that filter.")
        return self.random_element(keys)

    def restriction_enzyme_data(self) -> dict:
        """
         Returns data dict of a restriction enzyme. Does not work with 'faker.unique'
         as the dict is not hashable
        """
        return self._get_rest_dict()[self.restriction_enzyme()]

    def blunt(self) -> str:
        """
        Returns
        -------
        str
            Random blunt-end cutter.
        """
        keys = []
        for k, v in self._get_rest_dict().items():
            if v["ovhg"] is None or v["ovhg"] == 0:
                keys.append(k)
        return self.random_element(keys)

    def sticky(self) -> str:
        """
        Returns
        -------
        str
            Random sticky-ended cutter.
        """
        keys = []
        for k, v in self._get_rest_dict().items():
            if v["ovhg"] is not None and v["ovhg"] > 0:
                keys.append(k)
        return self.random_element(keys)


class Enzyme(BioProvider):
    """
     Provider of enzyme names. Source of data is Wikipedia:
         https://en.wikipedia.org/wiki/List_of_enzymes
             
    """

    def __init__(self, generator):
        super().__init__(generator)

    def categories(self) -> Sequence[str]:
        """
        A list of enzyme categories
        """

        return list(enzymes.keys())

    def _all_enzymes(self) -> list:
        ## cache
        if not hasattr(self, "_enzymes"):
            leaves = []
            self._dict_leaves(enzymes, leaves)
            self._enzymes = list(filter(lambda e: not e.startswith("Category"), leaves))
        return self._enzymes

    def enzyme_category(self) -> str:
        if not hasattr(self, "_categories"):
            all_vals = []
            self._dict_all(enzymes, all_vals)
            _categories = list(filter(lambda e: e.startswith("Category"), all_vals))
            self._categories = [x[9:] for x in _categories if x.startswith("Category:")]
        return self.random_element(self._categories)

    def enzyme(self) -> str:
        """
        Gets a random enzyme name
        """
        return self.random_element(self._all_enzymes())



from faker.providers import BaseProvider
import faker_biology.taxonomy.taxonomy_data as td


class ModelOrganism(BaseProvider):
    """
     Provider of random model organism names used in life-science research. 
    """

    def __init__(self, generator):
        super().__init__(generator)

    def organism_english(self) -> str:
        """
        English name of a  model organism used in life-science research
        Returns
        -------
        str
            A random model organism name.
        """
        return self.random_element(td.MODEL_ORGANISMS_ENGLISH)

    def organism_latin(self) -> str:
        """
        Latin name of a model organism used in life-science research
        Returns
        -------
        str
            A random model organism Latin name.
        """
        return self.random_element(td.MODEL_ORGANISMS_LATIN)
    
    def organism(self) -> td.MODEL_ORGANISM:
        """
        Object encapsulating data about a model organism used in life-science research
        Returns
        -------
        ModelOrganism
            A named tuple including  all information about the model organism.
        """
        return self.random_element(td.MODEL_ORGANISMS)



enzymes = {
    "Category:Oxidoreductases": {
        "Dehydrogenase": {},
        "Luciferase": {},
        "DMSO reductase": {},
        "Category:EC 1.1": {
            "Category:EC 1.1.1": {
                "Alcohol dehydrogenase (NAD)": {},
                "Alcohol dehydrogenase (NADP)": {},
                "Homoserine dehydrogenase": {},
                "Aminopropanol oxidoreductase": {},
                "Diacetyl reductase": {},
                "Glycerol dehydrogenase": {},
                "Propanediol-phosphate dehydrogenase": {},
                "glycerol-3-phosphate dehydrogenase (NAD+)": {},
                "D-xylulose reductase": {},
                "L-xylulose reductase": {},
                "Lactate dehydrogenase": {},
                "Malate dehydrogenase": {},
                "Isocitrate dehydrogenase": {},
                "HMG-CoA reductase": {},
            },
            "Category:EC 1.1.2": {},
            "Category:EC 1.1.3": {
                "Glucose oxidase": {},
                "L-gulonolactone oxidase": {},
                "Thiamine oxidase": {},
                "Xanthine oxidase": {},
            },
            "Category:EC 1.1.4": {},
            "Category:EC 1.1.5": {},
            "Category:EC 1.1.99": {},
        },
        "Category:EC 1.2": {
            "Category:EC 1.2.1": {
                "Acetaldehyde dehydrogenase": {},
                "Glyceraldehyde 3-phosphate dehydrogenase": {},
                "Pyruvate dehydrogenase": {},
            },
            "Category:EC 1.2.4": {"Oxoglutarate dehydrogenase": {}},
        },
        "Category:EC 1.3": {
            "Category:EC 1.3.1": {"Biliverdin reductase": {}},
            "Category:EC 1.3.2": {},
            "Category:EC 1.3.3": {"Protoporphyrinogen oxidase": {}},
            "Category:EC 1.3.5": {},
            "Category:EC 1.3.7": {},
            "Category:EC 1.3.99": {},
        },
        "Category:EC 1.4": {"Category:EC 1.4.3": {"Monoamine oxidase": {}}},
        "Category:EC 1.5": {
            "Category:EC 1.5.1": {
                "Dihydrofolate reductase": {},
                "Methylenetetrahydrofolate reductase": {},
            },
            "Category:EC 1.5.3": {
                "Sarcosine oxidase": {},
                "Dihydrobenzophenanthridine oxidase": {},
            },
            "Category:EC 1.5.4": {},
            "Category:EC 1.5.5": {},
            "Category:EC 1.5.7": {},
            "Category:EC 1.5.8": {},
            "Category:EC 1.5.99": {},
        },
        "Category:EC 1.6": {
            "Category:EC 1.6.1": {},
            "Category:EC 1.6.2": {},
            "Category:EC 1.6.3": {},
            "Category:EC 1.8.1": {},
            "Category:EC 1.6.5": {"NADH dehydrogenase": {}},
            "Category:EC 1.6.6": {},
            "Category:EC 1.18.1": {},
            "Category:EC 1.5.1": {},
            "Category:EC 1.6.99": {},
        },
        "Category:EC 1.7": {
            "Category:EC 1.7.1": {},
            "Category:EC 1.7.2": {},
            "Category:EC 1.7.3": {"Urate oxidase": {}},
            "Category:EC 1.7.7": {},
            "Category:EC 1.7.99": {"Nitrite reductase": {}, "Nitrate reductase": {}},
        },
        "Category:EC 1.8": {
            "Category:EC 1.8.1": {
                "Glutathione reductase": {},
                "Thioredoxin reductase": {},
            },
            "Category:EC 1.8.2": {},
            "Category:EC 1.8.3": {"Sulfite oxidase": {}},
            "Category:EC 1.8.4": {},
            "Category:EC 1.8.5": {},
            "EC 2.5.1.18": {},
            "Category:EC 1.8.7": {},
            "Category:EC 1.8.98": {},
            "Category:EC 1.8.99": {},
        },
        "Category:EC 1.9": {
            "Category:EC 1.9.3": {"Cytochrome c oxidase": {}},
            "Category:EC 1.9.6": {},
            "EC 1.9.98.1": {},
        },
        "Category:EC 1.10": {
            "Category:EC 1.10.1": {},
            "Category:EC 1.10.2": {"Coenzyme Q - cytochrome c reductase": {}},
            "Category:EC 1.10.3": {"Catechol oxidase": {}, "Laccase": {}},
            "Category:EC 1.10.99": {},
        },
        "Category:EC 1.11": {
            "Category:EC 1.11.1": {
                "Cytochrome c peroxidase": {},
                "Catalase": {},
                "Myeloperoxidase": {},
                "Thyroid peroxidase": {},
                "Glutathione peroxidase": {},
            }
        },
        "Category:EC 1.12": {
            "Category:EC 1.12.1": {},
            "Category:EC 1.12.2": {},
            "Category:EC 1.12.5": {},
            "Category:EC 1.12.7": {},
            "Category:EC 1.12.98": {},
            "Category:EC 1.12.99": {},
        },
        "Category:EC 1.13": {
            "Category:EC 1.13.11": {"4-hydroxyphenylpyruvate dioxygenase": {}},
            "Category:EC 1.13.12": {
                "Renilla-luciferin 2-monooxygenase": {},
                "Cypridina-luciferin 2-monooxygenase": {},
                "Firefly luciferase": {},
                "Watasenia-luciferin 2-monooxygenase": {},
                "Oplophorus-luciferin 2-monooxygenase": {},
            },
        },
        "Category:EC 1.14": {
            "Cytochrome P450 oxidase": {},
            "Category:Cytochrome P450": {
                "Aromatase": {},
                "CYP2D6": {},
                "CYP2E1": {},
                "CYP3A4": {},
                "Cytochrome P450 oxidase": {},
            },
            "Category:EC 1.14.12": {"Nitric oxide dioxygenase": {}},
            "Category:EC 1.14.13": {"Nitric oxide synthase": {}},
            "Category:EC 1.14.14": {
                "Aromatase": {},
                "CYP2D6": {},
                "CYP2E1": {},
                "CYP3A4": {},
            },
            "Category:EC 1.14.16": {"Phenylalanine hydroxylase": {}},
            "Category:EC 1.14.18": {"Tyrosinase": {}},
        },
        "Category:EC 1.15": {"Category:EC 1.15.1": {"Superoxide dismutase": {}}},
        "Category:EC 1.16": {"Category:EC 1.16.3": {"Ceruloplasmin": {}}},
        "Category:EC 1.17": {
            "Category:EC 1.17.1": {
                "Leucoanthocyanidin reductase": {},
                "Xanthine dehydrogenase": {},
                "Nicotinate dehydrogenase": {},
                "4-hydroxy-tetrahydrodipicolinate reductase": {},
            },
            "Category:EC 1.17.2": {"Nicotinate dehydrogenase (cytochrome)": {}},
            "Category:EC 1.17.3": {"Xanthine oxidase": {}},
            "Category:EC 1.17.4": {
                "Ribonucleotide reductase": {},
                "Ribonucleoside-triphosphate reductase": {},
                "Vitamin K epoxide reductase": {},
                "Vitamin-K-epoxide reductase (warfarin-sensitive)": {},
                "Vitamin-K-epoxide reductase (warfarin-insensitive)": {},
                "RRM1": {},
                "RRM2": {},
                "RRM2B": {},
            },
            "Category:EC 1.17.5": {"Caffeine dehydrogenase": {}},
            "Category:EC 1.17.7": {},
            "Category:EC 1.17.99": {},
        },
        "Category:EC 1.18": {"Category:EC 1.18.6": {"Nitrogenase": {}}},
        "Category:EC 1.19": {"Category:EC 1.19.6": {"Nitrogenase (flavodoxin)": {}}},
        "Category:EC 1.20": {
            "Category:EC 1.20.1": {},
            "Category:EC 1.20.2": {},
            "Category:EC 1.20.4": {
                "Arsenate reductase (glutaredoxin)": {},
                "Glutaredoxin": {},
            },
            "Category:EC 1.20.9": {},
            "Category:EC 1.20.99": {},
        },
        "Category:EC 1.21": {
            "Category:EC 1.21.1": {"Iodotyrosine deiodinase": {}},
            "Category:EC 1.21.3": {
                "Isopenicillin N synthase": {},
                "Tetrahydrocannabinolic acid synthase": {},
            },
            "Category:EC 1.21.4": {},
            "Category:EC 1.21.99": {
                "Thyroxine 5-deiodinase": {},
                "Iodothyronine deiodinase": {},
            },
        },
        "Category:EC 1.97": {"Category:EC 1.97.1": {"Deiodinase": {}}},
    },
    "Category:Transferases": {
        "Glutathione S-transferase": {},
        "Category:EC 2.1": {
            "Category:EC 2.1.1": {
                "Catechol-O-methyl transferase": {},
                "DNA methyltransferase": {},
                "Histone methyltransferase": {},
            },
            "Category:EC 2.1.3": {
                "Aspartate transcarbamoylase": {},
                "Ornithine transcarbamoylase": {},
            },
        },
        "Category:EC 2.2": {
            "Category:EC 2.2.1": {
                "Transketolase": {},
                "Transaldolase": {},
                "Acetolactate synthase": {},
                "2-Succinyl-5-enolpyruvyl-6-hydroxy-3-cyclohexene-1-carboxylic-acid synthase": {},
            }
        },
        "Category:EC 2.3": {
            "Category:EC 2.3.1": {
                "Aminolevulinic acid synthase": {},
                "Choline acetyltransferase": {},
            },
            "Category:EC 2.3.2": {
                "Factor XIII": {},
                "Gamma glutamyl transpeptidase": {},
                "Transglutaminase": {},
            },
        },
        "Category:EC 2.4": {
            "Category:EC 2.4.2": {"Hypoxanthine-guanine phosphoribosyltransferase": {}},
            "Category:EC 2.5": {},
            "Category:EC 2.5.1": {"Thiaminase": {}},
        },
        "Category:EC 2.5": {"Flavin prenyltransferase": {}},
        "Category:EC 2.6": {
            "Category:EC 2.6.1": {
                "Alanine transaminase": {},
                "Aspartate transaminase": {},
            }
        },
        "Category:EC 2.7": {"Category:EC 2.7.2": {"Butyrate kinase": {}}},
        "Category:EC 2.8": {
            "Thiosulfate sulfurtransferase": {},
            "3-mercaptopyruvate sulfurtransferase": {},
            "Thiosulfate--thiol sulfurtransferase": {},
            "tRNA uracil 4-sulfurtransferase": {},
            "Thiosulfate--dithiol sulfurtransferase": {},
            "Biotin synthase": {},
            "Cysteine desulfurase": {},
            "Lipoyl synthase": {},
            "Molybdenum cofactor sulfurtransferase": {},
            "Thiazole synthase": {},
            "Molybdopterin synthase sulfurtransferase": {},
            "Molybdopterin synthase": {},
            "tRNA-uridine 2-sulfurtransferase": {},
            "tRNA-5-taurinomethyluridine 2-sulfurtransferase": {},
            "tRNA-5-methyluridine(54) 2-sulfurtransferase": {},
        },
        "Category:EC 2.9": {
            "L-seryl-tRNA(Sec) selenium transferase": {},
            "O-phospho-L-seryl-tRNA(Sec):L-selenocysteinyl-tRNA synthase": {},
        },
    },
    "Category:Hydrolases": {
        "Hydrolytic enzyme": {},
        "Category:EC 3.1": {
            "Nuclease": {},
            "Endonuclease": {},
            "Exonuclease": {},
            "Category:EC 3.1.1": {
                "Acid hydrolase": {},
                "Phospholipase": {},
                "Acetylcholinesterase": {},
                "Cholinesterase": {},
                "Lipoprotein lipase": {},
            },
            "Category:EC 3.1.2": {"Ubiquitin carboxy-terminal hydrolase L1": {}},
            "Category:EC 3.1.3": {
                "Phosphatase": {},
                "Alkaline phosphatase": {},
                "Fructose bisphosphatase": {},
            },
            "Category:EC 3.1.4": {
                "Phospholipase": {},
                "CGMP specific phosphodiesterase type 5": {},
            },
            "Category:EC 3.1.21": {"Restriction enzyme": {}, "Deoxyribonuclease": {}},
            "Category:EC 3.1.26": {"RNase H": {}},
            "Category:EC 3.1.27": {"Ribonuclease": {}},
        },
        "Category:EC 3.2": {
            "Category:EC 3.2.1": {
                "Amylase": {},
                "Sucrase": {},
                "Chitinase": {},
                "Lysozyme": {},
                "Maltase": {},
                "Lactase": {},
                "Beta-galactosidase": {},
                "Hyaluronidase": {},
            }
        },
        "Category:EC 3.3": {
            "Category:EC 3.3": {},
            "Adenosylmethionine hydrolase": {},
            "S-adenosyl-L-homocysteine hydrolase": {},
            "Alkenylglycerophosphocholine hydrolase": {},
            "Alkenylglycerophosphoethanolamine hydrolase": {},
            "Cholesterol-5,6-oxide hydrolase": {},
            "Hepoxilin-epoxide hydrolase": {},
            "Isochorismatase": {},
            "Leukotriene-A4 hydrolase": {},
            "Limonene-1,2-epoxide hydrolase": {},
            "Microsomal epoxide hydrolase": {},
            "Trans-epoxysuccinate hydrolase": {},
        },
        "Category:EC 3.4": {
            "Category:EC 3.4.11": {"Alanine aminopeptidase": {}},
            "Category:EC 3.4.15": {"Angiotensin converting enzyme": {}},
            "Category:EC 3.4.21": {
                "Serine protease": {},
                "Chymotrypsin": {},
                "Trypsin": {},
                "Thrombin": {},
                "Factor X": {},
                "Plasmin": {},
                "Acrosin": {},
                "Factor VII": {},
                "Factor IX": {},
                "Prolyl oligopeptidase": {},
                "Factor XI": {},
                "Elastase": {},
                "Factor XII": {},
                "Proteinase K": {},
                "Tissue plasminogen activator": {},
                "Protein C": {},
            },
            "Category:EC 3.4.22": {"Separase": {}},
            "Category:EC 3.4.23": {
                "Pepsin": {},
                "Rennet": {},
                "Renin": {},
                "Trypsinogen": {},
                "Plasmepsin": {},
            },
            "Category:EC 3.4.24": {"Matrix metalloproteinase": {}},
            "Category:EC 3.4.25": {"Metalloendopeptidase": {}},
        },
        "Category:EC 3.5": {
            "Category:EC 3.5.1": {"Urease": {}},
            "Category:EC 3.5.2": {"Beta-lactamase": {}},
            "Category:EC 3.5.3": {"Arginase": {}},
            "Category:EC 3.5.4": {
                "Adenosine deaminase": {},
                "GTP cyclohydrolase I": {},
            },
            "Category:EC 3.5.5": {"Nitrilase": {}},
        },
        "Category:EC 3.6": {
            "Category:EC 3.6.1": {
                "Helicase": {},
                "DnaB helicase": {},
                "RecQ helicase": {},
            },
            "Category:EC 3.6.3": {"ATPase": {}, "NaKATPase": {}, "ATP synthase": {}},
        },
        "Category:EC 3.7": {"Kynureninase": {}},
        "Category:EC 3.8": {"Haloacetate dehalogenase": {}},
        "Category:EC 3.9": {
            "Phosphoamidase": {},
            "Protein arginine phosphatase": {},
            "Phosphohistidine phosphatase": {},
        },
        "Category:EC 3.10": {
            "N-sulfoglucosamine sulfohydrolase": {},
            "Cyclamate sulfohydrolase": {},
        },
        "Category:EC 3.11": {
            "Phosphonoacetaldehyde hydrolase": {},
            "Phosphonoacetate hydrolase": {},
            "Phosphonopyruvate hydrolase": {},
        },
        "Category:EC 3.12": {"Trithionate hydrolase": {}},
        "Category:EC 3.13": {
            "UDP-sulfoquinovose synthase": {},
            "2'-hydroxybiphenyl-2-sulfinate desulfinase": {},
            "3-sulfinopropanoyl-CoA desulfinase": {},
            "Carbon disulfide hydrolase": {},
            "(CysO sulfur-carrier protein)-S-L-cysteine hydrolase": {},
            "Carbonyl sulfide hydrolase": {},
            "S-adenosyl-L-methionine hydrolase (adenosine-forming)": {},
        },
    },
    "Category:Lyases": {
        "Category:EC 4.1": {
            "Category:EC 4.1.1": {
                "Ornithine decarboxylase": {},
                "Uridine monophosphate synthetase": {},
                "Aromatic-L-amino-acid decarboxylase": {},
                "RubisCO": {},
            },
            "Category:EC 4.1.2": {"Fructose-bisphosphate aldolase": {}},
        },
        "Category:EC 4.2": {
            "Category:EC 4.2.1": {"Carbonic anhydrase": {}, "Tryptophan synthase": {}}
        },
        "Category:EC 4.3": {"Category:EC 4.3.1": {"Phenylalanine ammonia-lyase": {}}},
        "Category:EC 4.4": {
            "Category:EC 4.4.1": {
                "Cystathionine gamma-lyase": {},
                "Cystathionine beta-lyase": {},
                "Leukotriene C4 synthase": {},
            }
        },
        "Category:EC 4.5": {
            "Category:EC 4.5.1": {
                "Dichloromethane dehalogenase": {},
                "Halohydrin dehalogenase": {},
            }
        },
        "Category:EC 4.6": {
            "Category:EC 4.6.1": {"Adenylate cyclase": {}, "Guanylate cyclase": {}}
        },
    },
    "Category:Isomerases": {
        "Category:EC 5.1": {
            "Category:EC 5.1.1": {},
            "Amino-acid racemase": {},
            "Serine racemase": {},
            "Category:EC 5.1.2": {},
            "Mandelate racemase": {},
            "Category:EC 5.1.3": {},
            "UDP-glucose 4-epimerase": {},
            "Category:EC 5.1.99": {},
            "Methylmalonyl CoA epimerase": {},
        },
        "Category:EC 5.2": {
            "Category:EC 5.2": {},
            "FKBP": {},
            "FKBP1B": {},
            "FKBP2": {},
            "FKBP3": {},
            "FKBP4": {},
            "FKBP5": {},
            "FKBP6": {},
            "FKBP8": {},
            "FKBP9": {},
            "FKBP10": {},
            "FKBPL": {},
            "Cyclophilin": {},
            "Parvulin": {},
            "Prolyl isomerase": {},
            "2-chloro-4-carboxymethylenebut-2-en-1,4-olide isomerase": {},
            "Beta-carotene isomerase": {},
            "Farnesol 2-isomerase": {},
            "Furylfuramide isomerase": {},
            "Linoleate isomerase": {},
            "Maleate isomerase": {},
            "Maleylacetoacetate isomerase": {},
            "Maleylpyruvate isomerase": {},
            "Photoisomerase": {},
            "Prolycopene isomerase": {},
            "Retinal isomerase": {},
            "Retinol isomerase": {},
            "Zeta-carotene isomerase": {},
        },
        "Category:EC 5.3": {
            "Category:EC 5.3.3": {"Enoyl CoA isomerase": {}},
            "Category:EC 5.3.4": {"Protein disulfide isomerase": {}},
        },
        "Category:EC 5.4": {"Category:EC 5.4.2": {"Phosphoglucomutase": {}}},
        "Category:EC 5.5": {},
        "Category:EC 5.99": {"Category:EC 5.99.1": {"Topoisomerase": {}}},
    },
    "Category:Ligases": {
        "Category:EC 6.1": {
            "6-carboxytetrahydropterin synthase": {},
            "Category:EC 6.1.1": {"FARSB": {}},
        },
        "Category:EC 6.2": {
            "Acetate--CoA ligase": {},
            "Medium-chain acyl-CoA ligase": {},
            "Long-chain-fatty-acid-CoA ligase": {},
            "Succinate-CoA ligase (GDP-forming)": {},
            "Succinate-CoA ligase (ADP-forming)": {},
            "Glutarate-CoA ligase": {},
            "Cholate-CoA ligase": {},
            "Oxalate-CoA ligase": {},
            "Malate-CoA ligase": {},
            "Acid-CoA ligase (GDP-forming)": {},
            "Biotin-CoA ligase": {},
            "4-coumarate-CoA ligase": {},
            "Acetate-CoA ligase (ADP-forming)": {},
            "6-carboxyhexanoate-CoA ligase": {},
            "Arachidonate-CoA ligase": {},
            "Acetoacetate-CoA ligase": {},
            "Propionate-CoA ligase": {},
            "Citrate-CoA ligase": {},
            "Long-chain-fatty-acid-luciferin-component ligase": {},
            "Long-chain-fatty-acid-(acyl-carrier-protein) ligase": {},
            "Transferred entry: 6.2.1.30": {},
            "(citrate (pro-3S)-lyase) ligase": {},
            "Dicarboxylate-CoA ligase": {},
            "Phytanate-CoA ligase": {},
            "Benzoate-CoA ligase": {},
            "o-succinylbenzoate-CoA ligase": {},
            "4-hydroxybenzoate-CoA ligase": {},
            "3a,7a-dihydroxy-5b-cholestanate-CoA ligase": {},
            "Transferred entry: 6.2.1.7": {},
            "Phenylacetate-CoA ligase": {},
            "2-furoate-CoA ligase": {},
            "Anthranilate-CoA ligase": {},
            "4-chlorobenzoate-CoA ligase": {},
            "Trans-feruloyl-CoA synthase": {},
            "ACP-SH:acetate ligase": {},
            "3-hydroxypropionyl-CoA synthase": {},
            "3-hydroxybenzoate\u2014CoA ligase": {},
            "(2,2,3-trimethyl-5-oxocyclopent-3-enyl)acetyl-CoA synthase": {},
            "(butirosin acyl-carrier protein)\u2014L-glutamate ligase": {},
            "4-hydroxybutyrate-CoA ligase": {},
            "3-((3aS,4S,7aS)-7a-methyl-1,5-dioxo-octahydro-1H-inden-4-yl)propanoate-CoA ligase": {},
            "3-oxocholest-4-en-26-oate-CoA ligase": {},
            "2-hydroxy-7-methoxy-5-methyl-1-naphthoate-CoA ligase": {},
            "3-(methylthio)propionyl-CoA ligase": {},
            "E1 ubiquitin-activating enzyme": {},
            "L-allo-isoleucine--holo-CmaA peptidyl-carrier protein ligase": {},
            "Medium-chain-fatty-acid-(acyl-carrier-protein) ligase": {},
            "Carnitine-CoA ligase": {},
            "Long-chain fatty acid adenylyltransferase FadD28": {},
            "4-hydroxybenzoate adenylyltransferase FadD22": {},
            "4-hydroxyphenylalkanoate adenylyltransferase FadD29": {},
            "L-firefly luciferin-CoA ligase": {},
            "L-proline-L-prolyl-carrier protein ligase": {},
            "D-alanine-D-alanyl-carrier protein ligase": {},
            "E1 SAMP-activating enzyme": {},
        },
        "Category:EC 6.3": {
            "Glutamine synthetase": {},
            "Argininosuccinate synthetase": {},
            "CTP synthase": {},
        },
        "Category:EC 6.4": {"Pyruvate carboxylase": {}, "Acetyl-CoA carboxylase": {}},
        "Category:EC 6.5": {"DNA ligase": {}},
        "Category:EC 6.6": {},
    },
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:27:03 2022

@author: richard
"""

import unittest
from faker import Faker
from faker_biology.physiology import Organelle


fake = Faker()
fake.add_provider(Organelle)
organ_provider = Organelle(None)


class OrganelleTest(unittest.TestCase):
 

    def test_random_unique_organelle(self):
        organ_set = set([fake.unique.organelle() for i in range(15)])
        self.assertEqual(15, len(organ_set))
        
    def test_common_organell(self):
        for _ in range (100):
         self.assertNotEqual(fake.common_eukaryotic_organelle(), "glycosome")
         
    def test_animal_organell(self):
        for _ in range (100):
            self.assertNotEqual(fake.animal_organelle(), "chloroplast")
            
    def test_plant_organell(self):
        for _ in range (100):
            self.assertNotEqual(fake.plant_organelle(), "lysosome")

    def test_random_organelle(self):
        print(fake.organelle())


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:27:03 2022

@author: richard
"""

import unittest
from faker import Faker
from faker_biology.mol_biol import Antibody


fake = Faker()
fake.add_provider(Antibody)
re_provider = Antibody(None)


class AntibodyTest(unittest.TestCase):
    def test_random_antibody_isotype(self):
        for i in range(10):
            fake.antibody_isotype()

    def test_random_antibody_application(self):
        for i in range(10):
            fake.antibody_application()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 16 18:03:13 2022

@author: richardadams
"""

ANTIBODY_ISOTYPE = [
    "IgG",
    "IgA",
    "IgD",
    "IgE",
    "IgG1",
    "IgG2",
    "IgG3",
    "IgG4",
    "IgM",
    "IgY",
]

ANTIBODY_APPLICATION = [
    "ChIP",
    "ELISA",
    "Flow Cytometry",
    "Immunocytochemistry",
    "Immunofluorescence",
    "Immunohistochemistry",
    "Western blot",
]

ANTIBODY_SOURCE = [
    "Chicken",
    "Goat",
    "Guinea Pig",
    "Human",
    "Mouse",
    "Rabbit",
    "Rat",
    "Sheep",
]

DILUTIONS = ["1:10", "1:50", "1:100", "1:250", "1:500", "1:1000"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 13 10:20:19 2022

@author: richard
"""


organ_data = {
    "Musculoskeletal system": {
        "Human skeleton": {},
        "Joints": {},
        "Ligaments": {},
        "Muscular system": {},
        "Tendons": {},
    },
    "Digestive system": {
        "Mouth": {"Teeth": {}, "Tongue": {}},
        "Salivary glands": {
            "Parotid glands": {},
            "Submandibular glands": {},
            "Sublingual glands": {},
        },
        "Pharynx": {},
        "Esophagus": {},
        "Stomach": {},
        "Small intestine": {"Duodenum": {}, "Jejunum": {}, "Ileum": {}},
        "Large intestine": {
            "Cecum": {},
            "Ascending colon": {},
            "Transverse colon": {},
            "Descending colon": {},
            "Sigmoid colon": {},
        },
        "Rectum": {},
        "Liver": {},
        "Gallbladder": {},
        "Mesentery": {},
        "Pancreas": {},
        "Anal canal": {},
    },
    "Respiratory system": {
        "Nasal cavity": {},
        "Pharynx": {},
        "Larynx": {},
        "Trachea": {},
        "Bronchi": {},
        "Bronchioles": {},
        "Lungs": {},
        "Muscles of breathing": {},
    },
    "Urinary system": {"Kidneys": {}, "Ureter": {}, "Bladder": {}, "Urethra": {}},
    "Female reproductive system": {
        "Internal reproductive organs": {
            "Ovaries": {},
            "Fallopian tubes": {},
            "Uterus": {},
            "Cervix": {},
            "Vagina": {},
        },
        "External reproductive organs": {"Vulva": {}, "Clitoris": {}},
        "Placenta": {},
    },
    "Male reproductive system": {
        "Internal reproductive organs": {
            "Testes": {},
            "Epididymis": {},
            "Vas deferens": {},
            "Seminal vesicles": {},
            "Prostate": {},
            "Bulbourethral glands": {},
        },
        "External reproductive organs": {"Penis": {}, "Scrotum": {}},
    },
    "Endocrine system": {
        "Pituitary gland": {},
        "Pineal gland": {},
        "Thyroid gland": {},
        "Parathyroid glands": {},
        "Adrenal glands": {},
        "Pancreas": {},
    },
    "Circulatory system": {"Heart": {}, "Arteries": {}, "Veins": {}, "Capillaries": {}},
    "Lymphatic system": {
        "Lymphatic vessel": {},
        "Lymph node": {},
        "Bone marrow": {},
        "Thymus": {},
        "Spleen": {},
        "Gut-associated lymphoid tissue": {"Tonsils": {}},
        "Interstitium": {},
    },
    "Nervous system": {
        "Brain": {"Cerebrum": {"Cerebral hemispheres": {}}, "Diencephalon": {}},
        "brainstem": {"Midbrain": {}, "Pons": {}, "Medulla oblongata": {}},
        "Cerebellum": {},
        "spinal cord": {},
        "ventricular system": {"Choroid plexus": {}},
    },
    "Peripheral nervous system": {
        "Nerves": {
            "Cranial nerves": {},
            "Spinal nerves": {},
            "Ganglia": {},
            "Enteric nervous system": {},
        }
    },
    "Sensory organs": {
        "Eye": {"Cornea": {}, "Iris": {}, "Ciliary body": {}, "Lens": {}, "Retina": {}},
        "Ear": {
            "Outer ear": {"Earlobe": {}},
            "Eardrum": {},
            "Middle ear": {"Ossicles": {}},
            "Inner ear": {
                "Cochlea": {},
                "Vestibule of the ear": {},
                "Semicircular canals": {},
            },
        },
        "Olfactory epithelium": {},
        "Tongue": {"Taste buds": {}},
    },
    "Integumentary system": {
        "Mammary glands": {},
        "Skin": {},
        "Subcutaneous tissue": {},
    },
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb 26 11:37:00 2022

@author: richard
"""
eukaryote_organelles = {
    "eukaryotes_major": [
        {"name": "cell membrane", "distribution": "all"},
        {"name": "cell wall", "distribution": "plants"},
        {"name": "chloroplast", "distribution": "plants"},
        {"name": "endoplasmic reticulum", "distribution": "all"},
        {"name": "flagellum", "distribution": "some"},
        {"name": "Golgi apparatus", "distribution": "all"},
        {"name": "mitochondrion", "distribution": "most"},
        {"name": "nucleus", "distribution": "all"},
        {"name": "vacuole", "distribution": "plants"},
    ],
    "eukaryotes_minor": [
        {"name": "acrosome", "distribution": "most"},
        {"name": "autophagosome", "distribution": "all"},
        {"name": "centriole", "distribution": "animals"},
        {"name": "cilium", "distribution": "most"},
        {"name": "cnidocyst", "distribution": "cnidarians"},
        {"name": "eyespot apparatus", "distribution": "algae"},
        {"name": "glycosome", "distribution": "protozoa"},
        {"name": "glyoxysome", "distribution": "plants"},
        {"name": "hydrogenosome", "distribution": "all"},
        {"name": "lysosome", "distribution": "animals"},
        {"name": "melanosome", "distribution": "animals"},
        {"name": "mitosome", "distribution": "unicellular"},
        {"name": "myofibril", "distribution": "animals"},
        {"name": "nucleolus", "distribution": "most"},
        {"name": "ocelloid", "distribution": "rare"},
        {"name": "parenthesome", "distribution": "fungi"},
        {"name": "peroxisome", "distribution": "all"},
        {"name": "porosome", "distribution": "all"},
        {"name": "ribosome", "distribution": "all"},
        {"name": "stress granule", "distribution": "most"},
        {"name": "vesicle", "distribution": "most"},
    ],
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb 12 10:05:04 2022

@author: richard
"""
import re, json, pprint
from collections import deque

## parses list of links. use pandoc to convert from mediawiki to gfm
## e.g. pandoc organs-wk-raw.py -f mediawiki -t gfm -o organs.md
## TODO handle titles and depth of lists depending on title
def parse_lines(lines):
    curr_depth = 0
    title_depth = 0
    line_index = 0
    stack = deque()
    for line in lines:
        ## ignore blank /whitespace
        if len(line) == 0 or line.isspace():
            continue
        depth, title_depth = get_depth2(line, title_depth)
        #  print(f"depth of {line} is {depth}")line
        line = line.rstrip("\n*[]").lstrip("#- ")
        match = re.findall(r"\[([^\[\]]+)\]", line)
        print(f"match is {match}")
        content = match[0] if len(match) > 0 else line
        if depth > curr_depth:
            stack.append({content: {}})
        elif depth == curr_depth:
            d = stack[-1]
            d[content] = {}
        elif depth < curr_depth:
            ## might be exiting a deeply nested structure back to top
            jump = curr_depth - depth
            print(content)
            pop_stack(stack, jump, 0)
            stack[-1][content] = {}

        curr_depth = depth
        line_index = line_index + 1
    jump = curr_depth
    pop_stack(stack, jump, 1)

    return stack[0]


def categories(data: dict):
    return list(data.keys())


def pop_stack(stack, diff: int, to: int):
    while diff != to:
        nested = stack.pop()
        ## uses fact all dicts are ordered by entry
        key = list(stack[-1].keys())[-1]
        stack[-1][key] = nested
        diff = diff - 1


def get_depth(line):
    m = re.match(r"(\*+)", line)
    return len(m.group())


def get_depth2(line, title_depth):
    if line.startswith("####"):
        return 3, 3
    elif line.startswith("###"):
        return 2, 2
    elif line.startswith("##"):
        return 1, 1
    else:
        #  print(line)
        m = re.match(r"^(\s+)", line)
        if m is None:
            return title_depth + 1, title_depth
        spaces = len(m.group())
        depth = spaces / 4
        return depth + title_depth + 1, title_depth


def dict_leaves(data: dict, leaves=[]):
    nodes = data.keys()
    for key in nodes:
        subnode = data[key]
        if len(subnode) > 0:
            dict_leaves(subnode, leaves)
        else:
            leaves.append(key)


def leaves_by_category(data: dict, category: str):
    leaves = []
    dict_leaves(data[category], leaves)
    return leaves


senses = {}
with open("enzymes.md") as data:
    lines = data.readlines()
    senses = parse_lines(lines)

    leaves = []
    dict_leaves(senses, leaves)
    print(leaves)
#    print(categories(senses))
# print(leaves_by_category(senses, "Eye"))
with open("enzymes.json", "w") as jsonw:
    jsonw.write(json.dumps(senses, indent=4))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 14 09:02:05 2022

@author: richard
"""


cell_types = {
    "Cells derived primarily from endoderm": {
        "Exocrine secretory epithelial cells.": {
            "Brunner's gland": {},
            "Goblet cell": {},
            "Stomach": {"Foveolar cell": {}, "Chief cell": {}, "Parietal cell": {}},
            "Pancreatic acinar": {},
            "Paneth cell": {},
            "Type II Pneumocyte": {},
            "Club cell": {},
        },
        "Barrier cells": {
            "Type I Pneumocyte": {},
            "Gall bladder epithelial cell": {},
            "Centroacinar cell": {},
            "Intercalated duct": {},
            "Intestinal brush border": {},
        },
        "Hormone-secreting cells": {
            "Enteroendocrine cell": {
                "gastric inhibitory peptide-secreting K cell": {},
                "glucagon-like peptide-1-secreting L cell": {},
                "cholecystokinin-secreting I cell": {},
                "gastrin-secreting G cell": {},
                "Enterochromaffin cell": {},
                "Enterochromaffin-like cell": {},
                "neurotensin-secreting N cell": {},
                "secretin-secreting S cell": {},
                "somatostatin-secreting D cell": {},
                "motilin-secreting M cel": {},
            },
            "Thyroid gland": {"Thyroid epithelial cell": {}, "Parafollicular cell": {}},
            "Parathyroid gland": {"Parathyroid chief cell": {}, "Oxyphil cell": {}},
            "Pancreatic islets": {
                "Alpha cell": {},
                "Beta cell": {},
                "Delta cell": {},
                "Epsilon cell": {},
                "PP cell": {},
            },
        },
    },
    "Cells derived primarily from ectoderm": {
        "Exocrine secretory epithelial cells": {
            "Salivary gland": {},
            "Von Ebner's gland": {},
            "Mammary gland": {},
            "Lacrimal gland cell": {},
            "Ceruminous gland": {},
            "Eccrine sweat gland": {},
            "Apocrine sweat gland": {},
            "Gland of Moll": {},
            "Sebaceous gland": {},
            "Bowman's gland": {},
        },
        "Hormone-secreting cells": {
            "pituitary": {
                "Corticotropes": {},
                "Gonadotropes": {},
                "Lactotropes": {},
                "Melanotropes": {},
                "Somatotropes": {},
                "Thyrotropes": {},
            },
            "Magnocellular neurosecretory cells": {},
            "Parvocellular neurosecretory cells": {"neurotensin": {}},
            "Chromaffin cells": {},
        },
        "Epithelial cells": {
            "Keratinocyte": {},
            "basal cell": {},
            "Melanocyte": {},
            "Trichocyte": {
                "Medullary": {},
                "Cortical": {},
                "Cuticular": {},
                "Huxley's layer": {},
                "Henle's layer": {},
                "Outer root sheath": {},
            },
            "Surface epithelial cell": {},
            "Basal cell (stem cell)": {},
            "Intercalated duct": {},
            "Striated duct": {},
            "Lactiferous duct": {},
            "Ameloblast": {},
        },
        "Oral cells": {"Odontoblast": {}, "Cementoblast": {}},
        "Nervous system": {
            "Sensory transducer cells": {
                "Auditory inner hair cells": {},
                "Auditory outer hair cells": {},
                "Basal cells of olfactory epithelium": {},
                "Cold-sensitive sensory neurons": {},
                "Heat-sensitive sensory neurons": {},
                "Merkel cells": {},
                "Olfactory receptor neurons": {},
                "Pain-sensitive sensory neurons": {},
                "Photoreceptor cells": {
                    "rod cells": {},
                    "blue-sensitive cone cells": {},
                    "red-sensitive cone cells": {},
                    "green-sensitive cone cells": {},
                },
                "Proprioceptive": {},
                "Touch-sensitive sensory neurons": {},
                "Chemoreceptor": {},
                "Outer hair cells": {},
                "Inner hair cells": {},
                "Taste receptor cells": {},
            },
            "Autonomic neuron cells": {
                "Cholinergic neurons": {},
                "Adrenergic neural cells": {},
                "Peptidergic neural cells": {},
            },
            "Sense organ and peripheral neuron supporting cells": {
                "Inner pillar cells of organ of Corti": {},
                "Outer pillar cells of organ of Corti": {},
                "Inner phalangeal cells of organ of Corti": {},
                "Outer phalangeal cells of organ of Corti": {},
                "Border cells of organ of Corti": {},
                "Hensen's cells": {},
                "Vestibular apparatus supporting cells": {},
                "Taste bud supporting cells": {},
                "Olfactory epithelium supporting cells": {},
                "Olfactory ensheathing cells": {},
                "Schwann cells": {},
                "Satellite glial cells": {},
                "Enteric glial cells": {},
            },
            "glial cells": {
                "Neuron": {
                    "Interneurons": {
                        "Basket cells": {},
                        "Cartwheel cells": {},
                        "Stellate cells": {},
                        "Golgi cells": {},
                        "Granule cells": {},
                        "Lugaro cells": {},
                        "Unipolar brush cells": {},
                        "Martinotti cells": {},
                        "Chandelier cells": {},
                        "Cajal\u2013Retzius cells": {},
                        "Double-bouquet cells": {},
                        "Neurogliaform cells": {},
                        "Retina horizontal cells": {},
                        "Amacrine cells": {"Starburst amacrine cells": {}},
                        "Spinal interneurons": {"Renshaw cells": {}},
                    },
                    "Principal cells": {
                        "Spindle neurons": {},
                        "Fork neurons": {},
                        "Pyramidal cells": {
                            "Place cells": {},
                            "Grid cells": {},
                            "Speed cells": {},
                            "Head direction cells": {},
                            "Betz cells": {},
                        },
                        "Stellate cells": {"Boundary cells": {}},
                        "Bushy cells": {},
                        "Purkinje cells": {},
                        "Medium spiny neurons": {},
                    },
                },
                "Astrocytes": {},
                "Oligodendrocytes": {},
                "Ependymal cells": {"Tanycytes": {}},
                "Pituicytes": {},
            },
            "Lens cells": {
                "Anterior lens epithelial cell": {},
                "Crystallin-containing lens fiber cell": {},
            },
        },
    },
    "Cells derived primarily from mesoderm": {
        "Metabolism and storage cells": {
            "Adipocytes": {"White fat cell": {}, "Brown fat cell": {}},
            "Liver lipocyte": {},
        },
        "Secretory cells": {
            "Cells of the Adrenal cortex": {
                "Zona glomerulosa": {},
                "Zona fasciculata": {},
                "Zona reticularis": {},
            },
            "Theca interna cell": {},
            "Progesterone-secreting coprpus luteum cells": {
                "Granulosa lutein cells": {},
                "Theca lutein cells": {},
            },
            "Leydig cell": {},
            "Seminal vesicle": {},
            "Prostate gland": {},
            "Bulbourethral gland": {},
            "Bartholin's gland": {},
            "Gland of Littre": {},
            "Uterus": {},
            "Juxtaglomerular cell": {},
            "Macula densa cell": {},
            "Peripolar cell": {},
            "Mesangial cell": {},
        },
        "Barrier cells": {
            "Urinary system": {
                "Parietal epithelial cell": {},
                "Podocyte": {},
                "Proximal tubule brush border cell": {},
                "Loop of Henle thin segment cell": {},
                "Kidney distal tubule cell": {},
                "Kidney collecting duct cell": {
                    "Principal cell": {},
                    "Intercalated cell": {},
                },
                "Transitional epithelium": {},
            },
            "Reproductive system": {
                "Duct cell of seminal vesicle": {},
                "Efferent ducts": {},
                "Epididymal principal cell": {},
                "Epididymal basal cell": {},
            },
            "Circulatory system": {"Endothelial cells": {}},
        },
        "Extracellular matrix cells": {
            "Planum semilunatum epithelial cell of vestibular system": {},
            "Organ of Corti": {},
            "Loose connective tissue": {},
            "Corneal": {},
            "Tendon fibroblasts": {},
            "Bone marrow": {},
            "Other non epithelial fibroblasts": {},
            "Pericyte": {"Hepatic stellate cell": {}},
            "Nucleus pulposus cell": {},
            "Hyaline cartilage": {},
            "Fibrocartilage": {},
            "Elastic cartilage": {},
            "Osteoblast": {},
            "Osteoprogenitor cell": {},
            "Hyalocyte": {},
            "Stellate cell": {},
            "Pancreatic stellate cell": {},
        },
        "Contractile cells": {
            "Skeletal muscle": {
                "Red skeletal muscle cell": {},
                "White skeletal muscle cell": {},
                "Intermediate skeletal muscle cell": {},
                "Nuclear bag cell": {},
                "Nuclear chain cell": {},
                "Myosatellite cell": {},
            },
            "Cardiac muscle": {
                "Cardiac muscle cell": {},
                "SA node cell": {},
                "Purkinje fiber": {},
            },
            "Smooth muscle cell": {},
            "Myoepithelial cell": {},
        },
        "Blood and immune system cells": {
            "Erythrocyte": {},
            "Megakaryocyte": {},
            "Platelets": {},
            "Monocyte": {},
            "Connective-tissue macrophage": {},
            "Epidermala Langerhans cell": {},
            "Osteoclast": {},
            "Dendritic cell": {},
            "Microglial cell": {},
            "Neutrophil granulocyte": {},
            "Eosinophil granulocyte": {},
            "Basophil granulocyte": {},
            "Mast cell": {},
            "Helper T cell": {},
            "Regulatory T cell": {},
            "Cytotoxic T cell": {},
            "Natural killer T cell": {},
            "B cell": {},
            "Plasma cell": {},
            "Natural killer cell": {},
            "Hematopoietic stem cells": {},
        },
        "Germ cells (primordially not)": {
            "Oogonium": {},
            "Spermatid": {},
            "Spermatocyte": {},
            "Spermatogonium cell": {},
            "Spermatozoon": {},
        },
        "Nurse cell": {
            "Granulosa cell": {},
            "Sertoli cell": {},
            "Epithelial reticular cell": {},
        },
        "Interstitial cells": {"Interstitial kidney cells": {}},
    },
}
